from spacy.tokens.doc import Doc
from spacy.tokens.token import Token
from spacy.language import Language

from spacy_wordnet.wordnet_domains import Wordnet, load_wordnet_domains

try:

    @Language.factory("spacy_wordnet", default_config={"lang": "en"})
    def wordnet_annotator(nlp, name, lang: str):
        return WordnetAnnotator(lang=lang)


except AttributeError:
    pass  # spacy 2.x


class WordnetAnnotator(object):
    __FIELD = "wordnet"

    def __init__(self, lang: str = "es"):
        Token.set_extension(WordnetAnnotator.__FIELD, default=None, force=True)
        load_wordnet_domains()
        self.__lang = lang

    def __call__(self, doc: Doc):
        for token in doc:
            wordnet = Wordnet(token=token, lang=self.__lang)
            token._.set(WordnetAnnotator.__FIELD, wordnet)

        return doc



from nltk.corpus import wordnet as wn
from spacy.tokens.token import Token

from spacy_wordnet.__utils__ import *

__WN_DOMAINS_PATH = os.path.join(get_package_basepath(), 'data/wordnet_domains.txt')

__WN_DOMAINS_BY_SSID = defaultdict(list)


def wordnet_domains_path() -> str:
    return __WN_DOMAINS_PATH

def load_wordnet_domains(path: Optional[str] = wordnet_domains_path()):
    if __WN_DOMAINS_BY_SSID:
        return

    for line in open(path, 'r'):
        ssid, domains = line.strip().split('\t')
        __WN_DOMAINS_BY_SSID[ssid] = domains.split(' ')


def get_domains_for_synset(synset: Synset) -> List[str]:
    ssid = '{}-{}'.format(str(synset.offset()).zfill(8), synset.pos())
    return __WN_DOMAINS_BY_SSID.get(ssid, [])

class Wordnet(object):

    def __init__(self, token: Token, lang: str = 'es'):
        self.__token = token
        self.__lang = fetch_wordnet_lang(lang)
        self.__synsets = self.__find_synsets(token, self.__lang)
        self.__lemmas = self.__find_lemmas()
        self.__wordnet_domains = self.__find_wordnet_domains()

    def synsets(self):
        return self.__synsets

    def lemmas(self):
        return self.__lemmas

    def wordnet_domains(self):
        return self.__wordnet_domains

    def wordnet_domains_for_synset(self, synset: Synset):
        return get_domains_for_synset(synset)

    def wordnet_synsets_for_domain(self, domains: List[str]):
        return [synset for synset in self.synsets() if self.__has_domains(synset, domains)]

    @staticmethod
    def __find_synsets(token: Token, lang: str):
        word_variants = [token.text]
        if token.pos in [VERB, NOUN, ADJ]:
            # extend synset coverage using lemmas
            word_variants.append(token.lemma_)

        for word in word_variants:
            token_synsets = wn.synsets(word, pos=spacy2wordnet_pos(token.pos), lang=lang)
            if token_synsets:
                return token_synsets

        return []

    @staticmethod
    def __has_domains(synset: Synset, domains: List[str]) -> bool:
        return not set(domains).isdisjoint(get_domains_for_synset(synset))

    def __find_wordnet_domains(self):
        return [domain for synset in self.synsets() for domain in get_domains_for_synset(synset)]

    def __find_lemmas(self):
        return [lemma for synset in self.__synsets for lemma in synset.lemmas(lang=self.__lang)]



from collections import defaultdict
from itertools import chain

from spacy_wordnet.__utils__ import _load_wordnet_domains_from_ppv, _load_wordnet_domains_from_txt, \
    _persist_wordnet_domains

ppv_path = 'wordnet_domains/*.ppv'
txt_path = 'wordnet_domains/*.txt'


def run():
    domains_ppv = _load_wordnet_domains_from_ppv(ppv_path)
    domains_txt = _load_wordnet_domains_from_txt(txt_path)
    domains = defaultdict(set)
    for k, v in chain(domains_ppv.items(), domains_txt.items()):
        for domain in v:
            domains[k].add(domain)

    _persist_wordnet_domains('wordnet_domains.txt', domains)


if __name__ == '__main__':
    run()



import unittest
from collections import defaultdict

import spacy

import numpy as np

from itertools import product

from spacy_wordnet.wordnet_annotator import WordnetAnnotator


class WordnetAnnotatorTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super().__init__(*args, **kwargs)

        self.nlp_en = spacy.load("en_core_web_sm")
        self.nlp_es = spacy.load("es_core_news_sm")

        try:
            # Add wordnet component
            self.nlp_en.add_pipe("spacy_wordnet", config={"lang": self.nlp_en.lang})
            self.nlp_es.add_pipe("spacy_wordnet", config={"lang": self.nlp_es.lang})
        except TypeError:  # spacy 2.x
            self.nlp_en.add_pipe(WordnetAnnotator(self.nlp_en.lang))
            self.nlp_es.add_pipe(WordnetAnnotator(self.nlp_es.lang))

    def test_english_annotations(self):

        token = self.nlp_en("contracts")[0]

        assert token._.wordnet.synsets()
        assert token._.wordnet.lemmas()
        assert token._.wordnet.wordnet_domains()

    def test_generate_variants_from_domain_list(self):

        economy_domains = ["finance", "banking"]
        enriched_sentence = []

        sentence = self.nlp_en("I want to withdraw 5,000 euros")

        for token in sentence:
            synsets = token._.wordnet.wordnet_synsets_for_domain(economy_domains)

            if synsets:
                lemmas_for_synset = []
                for s in synsets:
                    lemmas_for_synset.extend(s.lemma_names())
                enriched_sentence.append(
                    "({})".format("|".join(set(lemmas_for_synset)))
                )
            else:
                enriched_sentence.append(token.text)
        print(" ".join(enriched_sentence))

    def test_spanish_annotations(self):
        token = self.nlp_es("contratos")[0]

        assert token._.wordnet.synsets()
        assert token._.wordnet.lemmas()
        assert token._.wordnet.wordnet_domains()



import glob
import os
from collections import defaultdict
from typing import Optional, List, Dict, Set

from nltk.corpus.reader.wordnet import \
    ADJ as WN_ADJ, \
    ADV as WN_ADV, \
    NOUN as WN_NOUN, \
    VERB as WN_VERB, Synset

from spacy.parts_of_speech import ADJ, ADV, NOUN, VERB, AUX

from spacy_wordnet import get_package_basepath

__DEFAULT_LANG = 'spa'
__WN_LANGUAGES_MAPPING = dict(es=__DEFAULT_LANG, en='eng', pt='por')
__WN_POS_MAPPING = {
    ADJ: WN_ADJ,
    NOUN: WN_NOUN,
    ADV: WN_ADV,
    VERB: WN_VERB,
    AUX: WN_VERB
}


def spacy2wordnet_pos(spacy_pos: int) -> Optional[str]:
    return __WN_POS_MAPPING.get(spacy_pos)


def fetch_wordnet_lang(lang: Optional[str] = None) -> str:
    language = __WN_LANGUAGES_MAPPING.get(lang, None)

    if not language:
        raise Exception('Language {} not supported'.format(lang))

    return language


def _persist_wordnet_domains(path: str, domains: Dict[str, Set[str]]):
    with open(path, 'w') as file:
        file.writelines(['{}\n'.format('\t'.join([k, ' '.join(v)])) for k, v in domains.items()])
        file.close()


def _load_wordnet_domains_from_ppv(path: str, threshold: float = 0.00009) -> Dict[str, List[str]]:
    def domain_name_from_filename(filename: str) -> str:
        name, _ = os.path.splitext(os.path.basename(filename))
        return name

    domains_by_ssid = defaultdict(list)

    for filename in glob.glob(path):
        domain_name = domain_name_from_filename(filename)
        for line in open(filename, 'r'):
            ssid, weight = line.strip().split('\t')
            if float(weight) >= threshold:
                domains_by_ssid[ssid].append(domain_name)

    return domains_by_ssid


def _load_wordnet_domains_from_txt(filepath: str) -> Dict[str, List[str]]:
    domains_by_ssid = defaultdict(list)
    for filename in glob.glob(filepath):
        for line in open(filename, 'r'):
            row = line.strip().split(' ')
            ssid = row[0]
            domains = row[2:]
            domains_by_ssid[ssid].extend(domains)

    return domains_by_ssid



import spacy

from spacy_wordnet.wordnet_annotator import WordnetAnnotator

if __name__ == '__main__':
    import nltk
    nltk.download('wordnet')

    # Load an spacy model (supported models are "es" and "en")
    nlp = spacy.load('en')
    nlp.add_pipe("spacy_wordnet", after='tagger')
    token = nlp('prices')[0]

    # wordnet object link spacy token with nltk wordnet interface by giving acces to
    # synsets and lemmas
    token._.wordnet.synsets()
    token._.wordnet.lemmas()

    # And automatically tags with wordnet domains
    token._.wordnet.wordnet_domains()

    # Imagine we want to enrich the following sentence with synonyms
    sentence = nlp('I want to withdraw 5,000 euros')

    # spaCy WordNet lets you find synonyms by domain of interest
    # for example economy
    economy_domains = ['finance', 'banking']
    enriched_sentence = []

    # For each token in the sentence
    for token in sentence:
        # We get those synsets within the desired domains
        synsets = token._.wordnet.wordnet_synsets_for_domain(economy_domains)
        if not synsets:
            enriched_sentence.append(token.text)
        else:
            lemmas_for_synset = [lemma for s in synsets for lemma in s.lemma_names()]
            # If we found a synset in the economy domains
            # we get the variants and add them to the enriched sentence
            enriched_sentence.append('({})'.format('|'.join(set(lemmas_for_synset))))

    # Let's see our enriched sentence
    print(' '.join(enriched_sentence))
    # >> I (need|want|require) to (draw|withdraw|draw_off|take_out) 5,000 euros
